@@ -698,6 +698,7 @@ class ConjectureResult(object):
     output = attr.ib()
     extra_information = attr.ib()
     has_discards = attr.ib()
+    target_observations = attr.ib()
     forced_indices = attr.ib(repr=False)
     examples = attr.ib(repr=False)
 
@@ -752,9 +753,14 @@ class ConjectureData(object):
         self.draw_times = []
         self.max_depth = 0
         self.has_discards = False
+        self.consecutive_discard_counts = []
 
         self.__result = None
 
+        # Observations used for targeted search.  They'll be aggregated in
+        # ConjectureRunner.generate_new_examples and fed to TargetSelector.
+        self.target_observations = {}
+
         # Normally unpopulated but we need this in the niche case
         # that self.as_result() is Overrun but we still want the
         # examples for reporting purposes.
@@ -795,6 +801,7 @@ class ConjectureData(object):
                 if self.extra_information.has_information()
                 else None,
                 has_discards=self.has_discards,
+                target_observations=self.target_observations,
                 forced_indices=self.forced_indices,
             )
             self.blocks.transfer_ownership(self.__result)
@@ -862,15 +869,34 @@ class ConjectureData(object):
         if self.depth > self.max_depth:
             self.max_depth = self.depth
         self.__example_record.start_example(label)
+        self.consecutive_discard_counts.append(0)
 
     def stop_example(self, discard=False):
         if self.frozen:
             return
+        self.consecutive_discard_counts.pop()
         if discard:
             self.has_discards = True
         self.depth -= 1
         assert self.depth >= -1
         self.__example_record.stop_example(discard)
+        if self.consecutive_discard_counts:
+            # We block long sequences of discards. This helps us avoid performance
+            # problems where there is rejection sampling. In particular tests which
+            # have a very small actual state space but use rejection sampling will
+            # play badly with generate_novel_prefix() in DataTree, and will end up
+            # generating very long tests with long runs of the rejection sample.
+            if discard:
+                self.consecutive_discard_counts[-1] += 1
+                # 20 is a fairly arbitrary limit chosen mostly so that all of the
+                # existing tests passed under it. Essentially no reasonable
+                # generation should hit this limit when running in purely random
+                # mode, but unreasonable generation is fairly widespread, and our
+                # manipulation of the bitstream can make it more likely.
+                if self.consecutive_discard_counts[-1] > 20:
+                    self.mark_invalid()
+            else:
+                self.consecutive_discard_counts[-1] = 0
 
     def note_event(self, event):
         self.events.add(event)
@@ -943,15 +969,6 @@ class ConjectureData(object):
         assert bit_length(result) <= n
         return result
 
-    @property
-    def block_starts(self):
-        while self.__block_starts_calculated_to < len(self.blocks):
-            i = self.__block_starts_calculated_to
-            self.__block_starts_calculated_to += 1
-            u, v = self.blocks.bounds(i)
-            self.__block_starts[v - u].append(u)
-        return self.__block_starts
-
     def draw_bytes(self, n):
         """Draw n bytes from the underlying source."""
         return int_to_bytes(self.draw_bits(8 * n), n)
