@@ -39,7 +39,7 @@ LABEL_MASK = 2 ** 64 - 1
 
 
 def calc_label_from_name(name):
-    hashed = hashlib.md5(str_to_bytes(name)).digest()
+    hashed = hashlib.sha384(str_to_bytes(name)).digest()
     return int_from_bytes(hashed[:8])
 
 
@@ -167,7 +167,7 @@ def boolean(data):
 
 
 def biased_coin(data, p):
-    """Return False with probability p (assuming a uniform generator),
+    """Return True with probability p (assuming a uniform generator),
     shrinking towards False."""
     data.start_example(BIASED_COIN_LABEL)
     while True:
