@@ -25,6 +25,7 @@ import functools
 import inspect
 import io
 import numbers
+import sys
 import uuid
 
 import hypothesis.strategies as st
@@ -188,14 +189,35 @@ _global_type_lookup = {
     numbers.Number: st.complex_numbers(),
     numbers.Integral: st.integers(),
     numbers.Complex: st.complex_numbers(),
+    slice: st.builds(
+        slice,
+        st.none() | st.integers(),
+        st.none() | st.integers(),
+        st.none() | st.integers(),
+    )
     # Pull requests with more types welcome!
 }
 
 if PY2:
+    # xrange's |stop - start| must fit in a C long
+    int64_strat = st.integers(-sys.maxint // 2, sys.maxint // 2)  # noqa
     _global_type_lookup.update(
         {
             int: st.integers().filter(lambda x: isinstance(x, int)),
             long: st.integers().map(long),  # noqa
+            xrange: st.integers(min_value=0, max_value=sys.maxint).map(xrange)  # noqa
+            | st.builds(xrange, int64_strat, int64_strat)  # noqa
+            | st.builds(
+                xrange, int64_strat, int64_strat, int64_strat.filter(bool)  # noqa
+            ),
+        }
+    )
+else:
+    _global_type_lookup.update(
+        {
+            range: st.integers(min_value=0).map(range)
+            | st.builds(range, st.integers(), st.integers())
+            | st.builds(range, st.integers(), st.integers(), st.integers().filter(bool))
         }
     )
 
@@ -216,6 +238,9 @@ try:  # pragma: no cover
         array_shapes,
         scalar_dtypes,
         nested_dtypes,
+        from_dtype,
+        integer_dtypes,
+        unsigned_integer_dtypes,
     )
 
     _global_type_lookup.update(
@@ -225,7 +250,7 @@ try:  # pragma: no cover
         }
     )
 except ImportError:  # pragma: no cover
-    pass
+    np = None
 
 try:
     import typing
@@ -235,13 +260,15 @@ else:
     _global_type_lookup.update(
         {
             typing.ByteString: st.binary(),
-            typing.io.BinaryIO: st.builds(io.BytesIO, st.binary()),  # type: ignore
-            typing.io.TextIO: st.builds(io.StringIO, st.text()),  # type: ignore
             typing.Reversible: st.lists(st.integers()),
             typing.SupportsAbs: st.complex_numbers(),
             typing.SupportsComplex: st.complex_numbers(),
             typing.SupportsFloat: st.floats(),
             typing.SupportsInt: st.floats(),
+            # xIO are only available in .io on Python 3.5, but available directly
+            # as typing.*IO from 3.6 onwards and mypy 0.730 errors on the compat form.
+            typing.io.BinaryIO: st.builds(io.BytesIO, st.binary()),  # type: ignore
+            typing.io.TextIO: st.builds(io.StringIO, st.text()),  # type: ignore
         }
     )
 
@@ -251,6 +278,13 @@ else:
         _global_type_lookup[typing.SupportsRound] = st.complex_numbers()
     except AttributeError:  # pragma: no cover
         pass
+    try:
+        strat = st.integers() | st.booleans()
+        if np is not None:  # pragma: no branch
+            strat |= (unsigned_integer_dtypes() | integer_dtypes()).flatmap(from_dtype)
+        _global_type_lookup[typing.SupportsIndex] = strat  # type: ignore
+    except AttributeError:  # pragma: no cover
+        pass
 
     def register(type_, fallback=None):
         if isinstance(type_, str):
